@@ -192,6 +192,7 @@
 
       public :: read_basis_specs
       public :: label2species
+      public :: basis_specs_reset
 
       private
 
@@ -1060,7 +1061,23 @@ C Sanity checks on values
                ind = ind + 1
                if (.not. basp%non_perturbative_polorbs) then
                   ! Placeholder empty shell for perturbative polarization orbital
-                  ls%shell(ind)%n=basp%ground_state%n(l)
+                  ! Cover the case in which there are lower-lying occupied states for this l
+                  ! For example, in a transition metal with 5d states, the polarization
+                  ! orbital for 6p should be 6d
+                  ! (In the current code this condition could trigger a fallback to
+                  ! a non-perturbative polarization orbital, but the following check is
+                  ! inserted for completeness)
+                  if (basp%ground_state%occupied(l)) then
+                     ls%shell(ind)%n=basp%ground_state%n(l) + 1
+                  else
+                     ls%shell(ind)%n=basp%ground_state%n(l)
+                  endif
+
+                  ! Further check for corner case of high-n diffusion orbs
+                  if (ls%shell(ind)%n < s%n) then
+                     ls%shell(ind)%n = s%n
+                  endif
+
                   ls%shell(ind)%l = ls%l
                   ls%shell(ind)%nzeta = 0
                   print "(a,i1,a1)", trim(basp%label)//
@@ -1070,7 +1087,16 @@ C Sanity checks on values
                else             !! non_perturbative_polorbs
                   ! Copy again to inherit relevant data
                   call copy_shell(source=s,target=ls%shell(ind))
-                  ls%shell(ind)%n = basp%ground_state%n(l)     !! earlier: -1   ! reset to find later
+                  ! Cover the case in which there are lower-lying occupied states for this l
+                  ! For example, in a transition metal with 5d states, the polarization
+                  ! orbital for 6p should be 6d
+
+                  if (basp%ground_state%occupied(l)) then
+                     ls%shell(ind)%n=basp%ground_state%n(l) + 1
+                  else
+                     ls%shell(ind)%n=basp%ground_state%n(l)
+                  endif
+
                   ls%shell(ind)%l = l
                   ls%shell(ind)%nzeta = s%nzeta_pol
                   ls%shell(ind)%nzeta_pol = 0
@@ -1082,6 +1108,14 @@ C Sanity checks on values
                   ls%shell(ind)%shell_being_polarized =>
      $                 polarized_shell_ptr
 
+                  ! Further check for corner case of high-n diffusion orbs
+                  if ( ls%shell(ind)%n <
+     &                 ls%shell(ind)%shell_being_polarized%n ) then
+                    ls%shell(ind)%n =
+     &                 ls%shell(ind)%shell_being_polarized%n
+                  endif
+
+
                   ! rc's, lambdas, etc, will remain as in s.
 
                   ! ... except that we honor any explicit charge-confinement settings
@@ -1891,4 +1925,9 @@ c (according to atmass subroutine).
 
       end subroutine autobasis
 
+      subroutine basis_specs_reset()
+        implicit none
+        if ( associated(basp) ) call destroy( basp )
+      end subroutine basis_specs_reset
+
       End module basis_specs
